@@ -42,17 +42,27 @@ The list of file names can be listed through:
 ar.archived_files.keys()
 
 Files themselves can be opened by getting the value of:
-f = ar.archived_files['filename']
+f = ar.archived_files[b'filename']
 
 and read through:
 f.read([length])
 
-random access through seek and tell functions is supported on the archived files
+random access through seek and tell functions is supported on the archived files.
+
+zipfile-like interface is also available:
+
+ar.namelist() will return a list of names (with possible duplicates)
+ar.infolist() will return a list of headers
+
+Use ar.open(name / header) to get the specific file.
+
+You can also use context manager syntax with either the ar file or its contents.
 """
 
 import io
 import struct
-from typing import Optional, List, Dict, BinaryIO, cast
+import os.path
+from typing import Optional, List, Dict, BinaryIO, cast, Union
 
 
 HEADER_BSD = 1
@@ -103,8 +113,14 @@ class ArchiveFileHeader(object):
 
 			if self.type in (HEADER_NORMAL, HEADER_BSD, HEADER_GNU):
 				self.timestamp = int(timestamp)
-				self.uid = int(uid)
-				self.gid = int(gid)
+				if uid.strip():
+					self.uid = cast(Optional[int], int(uid))
+				else:
+					self.uid = None
+				if gid.strip():
+					self.gid = cast(Optional[int], int(gid))
+				else:
+					self.gid = None
 				self.mode = int(mode, 8)
 
 		except ValueError as err:
@@ -180,6 +196,34 @@ class ArchiveFileData(io.IOBase):
 	def seekable(self) -> bool:
 		return self.arobj.seekable
 
+	def __enter__(self):
+		return self
+
+	def __exit__(self, _exc_type, _exc_value, _traceback):
+		return False
+
+class ArchiveFileDataThin(ArchiveFileData):
+	""" File-like object used for reading a thin archived file """
+
+	def __init__(self, ar_obj: "Archive", header: ArchiveFileHeader):
+				ArchiveFileData.__init__(self, ar_obj, header)
+				self.file_path=os.path.dirname(ar_obj.file.name)+ "/"+header.name.decode()
+
+
+	def read(self, size: Optional[int] = None) -> bytes:
+		""" Reads the data from the archived file, simulates file.read """
+		if size is None:
+			size = self.header.size - self.last_offset
+
+		with open(self.file_path, "rb") as f:
+			f.seek(self.last_offset)
+			data=f.read(size)
+
+		if len(data) < size:
+			raise ArchiveAccessError("incorrect archive file")
+		self.last_offset += size
+		return data
+
 class Archive(object):
 	""" Archive object allowing reading of *.ar files """
 
@@ -192,8 +236,14 @@ class Archive(object):
 		else:
 			raise ValueError("either filename or fileobj argument needs to be given")
 		self.position = 0
+		self.reached_eof = False
 		self._detect_seekable()
-		if self._read(GLOBAL_HEADER_LEN) != b"!<arch>\n":
+		global_header=self._read(GLOBAL_HEADER_LEN)
+		if global_header == b"!<arch>\n":
+			self.file_data_class = ArchiveFileData
+		elif global_header == b"!<thin>\n":
+			self.file_data_class = ArchiveFileDataThin
+		else:
 			raise ArchiveFormatError("file is missing the global header")
 
 		self.next_header_offset = GLOBAL_HEADER_LEN
@@ -227,6 +277,7 @@ class Archive(object):
 			while self.position < offset:
 				if not self._read(min(4096, offset - self.position)):
 					# reached EOF before target offset
+					self.reached_eof = True
 					return
 
 	def __read_file_header(self, offset: int) -> Optional[ArchiveFileHeader]:
@@ -236,6 +287,7 @@ class Archive(object):
 		header = self._read(HEADER_LEN)
 
 		if len(header) == 0:
+			self.reached_eof = True
 			return None
 		if len(header) < HEADER_LEN:
 			raise ArchiveFormatError("file header too short")
@@ -321,7 +373,7 @@ class Archive(object):
 		if header is not None:
 			self.headers.append(header)
 			if header.type in (HEADER_BSD, HEADER_NORMAL, HEADER_GNU):
-				self.archived_files[header.name] = ArchiveFileData(self, header)
+				self.archived_files[header.name] = self.file_data_class(self, header)
 
 		return header
 
@@ -339,9 +391,62 @@ class Archive(object):
 
 	def read_all_headers(self) -> None:
 		""" Reads all headers """
+		if self.reached_eof:
+			return
+
 		while self.read_next_header() is not None:
 			pass
 
 	def close(self) -> None:
 		""" Closes the archive file descriptor """
 		self.file.close()
+
+	### implement a zipfile-like interface as well
+
+	def namelist(self) -> List[bytes]:
+		"""
+		Return the names of files stored in the archive
+
+		If there are multiple files of the same name, there may be duplicates in the list.
+		"""
+		self.read_all_headers()
+		return [header.name for header in self.headers if header.type in (HEADER_BSD, HEADER_NORMAL, HEADER_GNU)]
+
+	def infolist(self) -> List[ArchiveFileHeader]:
+		"""
+		Return the headers of files stored in the archive
+
+		These can be used with .open() to get the contents.
+		"""
+		self.read_all_headers()
+		return [header for header in self.headers if header.type in (HEADER_BSD, HEADER_NORMAL, HEADER_GNU)]
+
+	def open(self, name: Union[bytes,ArchiveFileHeader]) -> ArchiveFileData:
+		"""
+		Return a file-like object based on the provided name or header
+
+		The name can be either a filename, or a header obtained from .read_next_header() or .infolist()
+		"""
+		self.read_all_headers()
+
+		if isinstance(name, bytes):
+			ar_file = self.archived_files.get(name)
+			if ar_file is None:
+				raise KeyError("There is no item named %r in the archive" % (name,))
+
+			return ar_file
+
+		if isinstance(name, ArchiveFileHeader):
+			if name not in self.headers:
+				raise KeyError("Provided header does not match this archive")
+
+			return ArchiveFileData(ar_obj=self, header=name)
+
+		raise ValueError("Can't look up file using type %s, expected bytes or ArchiveFileHeader" % (type(name),))
+
+	def __enter__(self):
+		return self
+
+	def __exit__(self, _exc_type, _exc_value, _traceback):
+		self.close()
+		return False
